#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""gtk_.gui.py"""

# Copyright (C) 2009, 2010, 2011, 2012, 2013 Federico Brega, Pierluigi Villani

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from __future__ import print_function
import os
import StringIO
import gtk
import pango
import webkit
import cairo
import glal
from themes import ThemeManager
from version import VERSION

pixmap_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                          '..','..',
                                          'pixmaps')
ICON_FILENAME = os.path.join(pixmap_dir,'cyclograph.svg')
if not os.path.isfile(ICON_FILENAME):
    pixmap_dir = '/usr/share/pixmaps/'
    ICON_FILENAME = '/usr/share/icons/hicolor/scalable/apps/cyclograph.svg'

class Gui(gtk.Window):
    """Main window"""
    def __init__(self, parent, title, size):
        super(Gui, self).__init__()

        self.cg_exit = None #check on exit callback
        self.set_title(title)
        self.maximized = False
        self.set_default_size(size[0], size[1])
        if size[2]:
            self.maximize()
            self.maximized = True
        try:
            self.set_icon_from_file(ICON_FILENAME)
        except Exception:
            pass #if librsvg2-common is not instaled
        self.set_position(gtk.WIN_POS_CENTER)

        self.notebook = gtk.Notebook()
        self.notebook.set_property('show-tabs', True)

        agr = gtk.AccelGroup()
        self.add_accel_group(agr)

        #Statusbar
        self.statusbar = gtk.Statusbar()
        self.context_id = self.statusbar.get_context_id("Statusbar")
        self.sbtimer = glal.Timer(5000, self.clearstatusbartext) #5000ms timeout

        #Load icons
        icons = [('icon_add', os.path.join(pixmap_dir, 'icon_add.png')),
                  ('icon_ckml', os.path.join(pixmap_dir, 'icon_ckml.png')),
                  ('icon_delete', os.path.join(pixmap_dir, 'icon_delete.png')),
                  ('icon_modify', os.path.join(pixmap_dir, 'icon_modify.png')),
                  ('icon_plot', os.path.join(pixmap_dir, 'icon_plot.png')),
                  ('icon_map', os.path.join(pixmap_dir, 'icon_map.png')),
                  ('icon_info', os.path.join(pixmap_dir, 'icon_info.png'))]

        factory = gtk.IconFactory()
        for (icon_id, icon_file) in icons:
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file(icon_file)
            except Exception:
                continue #if librsvg2-common is not instaled
            iconset = gtk.IconSet(pixbuf)
            factory.add(icon_id, iconset)
        factory.add_default()

        #Action group
        self.menu_item_new = gtk.Action("New", _(u"New"), _(u"New Slope"), gtk.STOCK_NEW)
        self.menu_item_open = gtk.Action("Open", _(u"Open"), _(u"Open Slope"), gtk.STOCK_OPEN)
        self.menu_item_createkml = gtk.Action("CreateKML", _(u"Create"), _(u"Create KML"), 'icon_ckml')
        self.menu_item_save = gtk.Action("Save", _(u"Save"), _(u"Save Slope"), gtk.STOCK_SAVE)
        
        actiongroup = gtk.ActionGroup("actions")
        actiongroup.add_action(self.menu_item_new)
        actiongroup.add_action(self.menu_item_open)
        actiongroup.add_action(self.menu_item_createkml)
        actiongroup.add_action(self.menu_item_save)

        #Toolbar
        toolbar = gtk.Toolbar()

        self.action_new = self.menu_item_new.create_tool_item()
        self.action_open = self.menu_item_open.create_tool_item()
        self.action_createkml = self.menu_item_createkml.create_tool_item()
        self.action_save = self.menu_item_save.create_tool_item()

        self.action_add = gtk.ToolButton('icon_add')
        self.action_add.set_label_widget(gtk.Label(_(u"Add")))
        self.action_add.set_homogeneous(False)
        self.action_add.set_tooltip_text(_(u"Add a check-point"))
        key, mod = gtk.accelerator_parse('plus')
        self.action_add.add_accelerator("clicked", agr,
                                                 key, mod, gtk.ACCEL_VISIBLE)
        self.action_edit = gtk.ToolButton('icon_modify')
        self.action_edit.set_label_widget(gtk.Label(_(u"Edit")))
        self.action_edit.set_homogeneous(False)
        self.action_edit.set_tooltip_text(_(u"Edit a check-point"))
        self.action_delete = gtk.ToolButton('icon_delete')
        self.action_delete.set_label_widget(gtk.Label(_(u"Delete")))
        self.action_delete.set_homogeneous(False)
        self.action_delete.set_tooltip_text(_(u"Remove a check-point"))
        key, mod = gtk.accelerator_parse('minus')
        self.action_delete.add_accelerator("clicked", agr,
                                                 key, mod, gtk.ACCEL_VISIBLE)
        self.action_plot = gtk.ToolButton('icon_plot')
        self.action_plot.set_homogeneous(False)
        self.action_plot.set_label_widget(gtk.Label(_(u"Plot")))
        self.action_plot.set_tooltip_text(_(u"Plot your slope"))
        self.action_map = gtk.ToolButton('icon_map')
        self.action_map.set_homogeneous(False)
        self.action_map.set_label_widget(gtk.Label(_(u"Map")))
        self.action_map.set_tooltip_text(_(u"Show the map of your slope"))
        self.action_properties = gtk.ToolButton('icon_info')
        self.action_properties.set_homogeneous(False)
        self.action_properties.set_label_widget(gtk.Label(_(u"Info")))
        self.action_properties.set_tooltip_text(_(u"Informations about the slope"))

        self.action_save.set_sensitive(False)
        self.action_add.set_sensitive(False)
        self.action_edit.set_sensitive(False)
        self.action_delete.set_sensitive(False)
        self.action_plot.set_sensitive(False)
        self.action_map.set_sensitive(False)
        self.action_properties.set_sensitive(False)

        toolbar.insert(self.action_new, 0)
        toolbar.insert(self.action_open, 1)
        toolbar.insert(self.action_createkml, 2)
        toolbar.insert(self.action_save, 3)
        toolbar.insert(gtk.SeparatorToolItem(), 4)
        toolbar.insert(self.action_add, 5)
        toolbar.insert(self.action_edit, 6)
        toolbar.insert(self.action_delete, 7)
        toolbar.insert(gtk.SeparatorToolItem(), 8)
        toolbar.insert(self.action_plot, 9)
        toolbar.insert(self.action_map, 10)
        toolbar.insert(self.action_properties, 11)

        #Creating the menubar.
        menubar = gtk.MenuBar()
        filemenu = gtk.Menu()
        self.smenu = gtk.MenuItem(_(u"_File"))
        self.smenu.set_submenu(filemenu)

        item_new = self.menu_item_new.create_menu_item()
        key, mod = gtk.accelerator_parse("<Control>N")
        item_new.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(item_new)

        sep = gtk.SeparatorMenuItem()
        filemenu.append(sep)

        item_open = self.menu_item_open.create_menu_item()
        key, mod = gtk.accelerator_parse("<Control>O")
        item_open.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(item_open)

        item_createkml = self.menu_item_createkml.create_menu_item()
        key, mod = gtk.accelerator_parse("<Control>G")
        item_createkml.add_accelerator("activate", agr,
                                                 key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(item_createkml)

        item_save = self.menu_item_save.create_menu_item()
        self.menu_item_save.set_sensitive(False)
        key, mod = gtk.accelerator_parse("<Control>S")
        item_save.add_accelerator("activate", agr,
                                            key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(item_save)

        self.menu_item_save_as = gtk.ImageMenuItem(gtk.STOCK_SAVE_AS, agr)
        self.menu_item_save_as.set_sensitive(False)
        key, mod = gtk.accelerator_parse("<Control><alt>S")
        self.menu_item_save_as.add_accelerator("activate", agr,
                                            key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(self.menu_item_save_as)

        sep = gtk.SeparatorMenuItem()
        filemenu.append(sep)


        self.menu_item_exit = gtk.ImageMenuItem(gtk.STOCK_QUIT, agr)
        key, mod = gtk.accelerator_parse("<Control>Q")
        self.menu_item_exit.add_accelerator("activate", agr,
                                            key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(self.menu_item_exit)

        menubar.append(self.smenu)

        filemenu1 = gtk.Menu()
        self.omenu = gtk.MenuItem(_(u"_Options"))
        self.omenu.set_submenu(filemenu1)

        kmenu = gtk.Menu()
        kmenum = gtk.MenuItem("_Kml server")
        kmenum.set_submenu(kmenu)

        self.menu_item_s1 = gtk.RadioMenuItem( None, (u"geonames.org"))
        self.menu_item_s2 = gtk.RadioMenuItem(self.menu_item_s1, (u"earthtools.org"))
        self.menu_item_s3 = gtk.RadioMenuItem(self.menu_item_s1, (u"usgs.net"))

        kmenu.append(self.menu_item_s1)
        kmenu.append(self.menu_item_s2)
        kmenu.append(self.menu_item_s3)

        filemenu1.append(kmenum)

        sep = gtk.SeparatorMenuItem()
        filemenu1.append(sep)

        self.preferences = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES)
        filemenu1.append(self.preferences)

        menubar.append(self.omenu)

        filemenu2 = gtk.Menu()
        self.amenu = gtk.MenuItem((u"?"))
        self.amenu.set_submenu(filemenu2)

        self.menu_item_about = gtk.ImageMenuItem(gtk.STOCK_ABOUT, agr)
        self.menu_item_about.connect("activate", onabout)
        filemenu2.append(self.menu_item_about)

        self.menu_item_bug = gtk.MenuItem(_(u"Signal a bug"))
        filemenu2.append(self.menu_item_bug)

        menubar.append(self.amenu)

        vbox = gtk.VBox(False, 0)
        vbox.pack_start(menubar, False, False, 0)
        vbox.pack_start(toolbar, False, False, 0)
        vbox.pack_start(self.notebook, True, True, 0)
        vbox.pack_start(self.statusbar, False, False, 0)

        self.add(vbox)
        self.connect("delete-event", self.exit)
        self.connect('window-state-event', self.on_window_state_event)
        self.show_all()

    def exit(self, event=None, param=None):
        """ Exit from CycloGraph, check if there are unsaved slopes"""
        if self.cg_exit():
            gtk.main_quit()
            return
        return True #needed by gtk

    def getdimensions(self):
        """Return window width and height"""
        return self.get_size()

    def ismaximized(self):
        """Return true if window is maximized"""
        return self.maximized

    def addstatusbartext(self, text):
        """Add text to the statusbar"""
        self.statusbar.push(self.context_id, text)

    def clearstatusbartext(self):
        """Remove text from statusbar"""
        self.statusbar.pop(self.context_id)
        self.sbtimer.stop()

    def on_window_state_event(self, widget, event):
            if event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
                self.maximized = True
            else:
                self.maximized = False

def onabout(event):
    """ About dialog"""
    about = gtk.AboutDialog()
    about.set_program_name("CycloGraph")
    about.set_version(VERSION)
    about.set_copyright("Copyright (C) 2008-2013 Federico Brega, Pierluigi Villani")
    about.set_authors(("Federico Brega", "Pierluigi Villani"))
    about.set_license("""
CycloGraph is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

CycloGraph is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see http://www.gnu.org/licenses/.
""")
    about.set_website("http://cyclograph.sourceforge.net")
    about.set_logo(gtk.gdk.pixbuf_new_from_file(ICON_FILENAME))
    about.set_icon_from_file(ICON_FILENAME)
    about.run()
    about.destroy()

class Page(gtk.ScrolledWindow):
    """Notebook page"""
    def __init__(self, parent):
        gtk.ScrolledWindow.__init__(self)
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        self.store = gtk.ListStore(str, int, str)
        self.mylistctrl = gtk.TreeView(self.store)
        self.mylistctrl.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.mylistctrl.set_rules_hint(True)       #rows of alternate color
        self.mylistctrl.set_rubber_banding(True)   #can select multiple cp by dragging the mouse
        self.add(self.mylistctrl)
        self.savedfilepath = None
        self.modified = False
        self.slopecounternew = 0
        self.closebox = None

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_(u"Distance [km]"), rendererText, text=0)
        column.set_sort_column_id(0)
        self.mylistctrl.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_(u"Altitude [m]"), rendererText, text=1)
        column.set_sort_column_id(1)
        self.mylistctrl.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_(u"Name"), rendererText, text=2)
        column.set_sort_column_id(2)
        self.mylistctrl.append_column(column)

        self.mylistctrl.set_headers_clickable(False) #disable column sort

    def insert_row(self, row_num, chkpnt):
        """Insert item to this page at position row_num"""
        self.store.insert(row_num, chkpnt)
    def delete_row(self, row_num):
        """Removes row at position row_num in this page."""
        itera = self.store.get_iter (path = (row_num,))
        self.store.remove (itera)
    def get_sel_row(self):
        """Gives the row number that is selected."""
        selection = self.mylistctrl.get_selection()
        model, selected = selection.get_selected_rows()
        editcp = -1
        if len(selected) > 0:
            path = selected[0]
            editcp = path[0]
        return editcp
    def get_sel_multi_row(self):
        """Gives a list with the row number of selected items."""
        selection = self.mylistctrl.get_selection()
        model, selected = selection.get_selected_rows()
        rm_cps = [ path[0] for path in selected]
        return rm_cps

class Managecp(gtk.Dialog):
    """Add and edit dialog"""
    def __init__(self, parent, title, dist="", alt="", name=""):
        gtk.Dialog.__init__(self, title, parent,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
                     (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

        self.set_size_request(220, 150)
        box = gtk.VBox(spacing=10)
        box.set_border_width(10)
        self.vbox.pack_start(box)
        box.show()

        labelbox = gtk.VBox(spacing=10)
        labelbox.set_border_width(10)
        labelbox.pack_start(gtk.Label(_(u"Distance:")))
        labelbox.pack_start(gtk.Label(_(u"Altitude:")))
        labelbox.pack_start(gtk.Label(_(u"Name:")))

        entrybox = gtk.VBox()
        self.distance = gtk.Entry()
        self.distance.set_text(dist)
        self.distance.set_activates_default(True)
        entrybox.pack_start(self.distance)
        self.altitude = gtk.Entry()
        self.altitude.set_text(alt)
        self.altitude.set_activates_default(True)
        entrybox.pack_start(self.altitude)
        self.cpname = gtk.Entry()
        self.cpname.set_text(name)
        self.cpname.set_activates_default(True)
        entrybox.pack_start(self.cpname)

        obox = gtk.HBox()
        obox.pack_start(labelbox, False, False, 0)
        obox.pack_start(entrybox)

        self.set_default_response(gtk.RESPONSE_ACCEPT)

        box.pack_start(obox)
        box.show_all()

    def show_modal(self):
        """Shows the dialog and returns True if the value has to be changed."""
        return self.run() == gtk.RESPONSE_ACCEPT
    def destroy(self):
        """Destroy current managecp"""
        gtk.Dialog.destroy(self)
    def getcp(self):
        """Gives the modified values."""
        dist = self.distance.get_text()
        alti = self.altitude.get_text()
        name = self.cpname.get_text()
        return (dist, alti, name)

class FormSlopeInfo(gtk.Dialog):
    """info dialog about the slope"""
    def __init__(self, parent, slopedata):
        gtk.Dialog.__init__(self, _(u"Slope Info"), parent,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
                     (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

        self.set_size_request(300, 420)
        box = gtk.VBox(spacing=10)
        box.set_border_width(10)
        self.vbox.pack_start(box)
        box.show()

        table = gtk.Table(8, 2, False)
        box.pack_start(table, False, True, 0)
        table.show()

        table.attach(gtk.Label(_(u"Name:")), 0, 1, 0, 1,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.s_name = gtk.Entry()
        self.s_name.set_text(slopedata['name'])
        table.attach(self.s_name, 1, 2, 0, 1,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"State:")), 0, 1, 1, 2,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.s_state = gtk.Entry()
        self.s_state.set_text(slopedata['state'])
        table.attach(self.s_state, 1, 2, 1, 2,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"Author:")), 0, 1, 2, 3,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.s_author = gtk.Entry()
        self.s_author.set_text(slopedata['author'])
        table.attach(self.s_author, 1, 2, 2, 3,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"E-mail:")), 0, 1, 3, 4,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.s_email = gtk.Entry()
        self.s_email.set_text(slopedata['email'])
        table.attach(self.s_email, 1, 2, 3, 4,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"Average gradient:")), 0, 1, 4, 5,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        table.attach(gtk.Label(slopedata['average_grad']), 1, 2, 4, 5,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"Max gradient:")), 0, 1, 5, 6,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        table.attach(gtk.Label(slopedata['max_grad']), 1, 2, 5, 6,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"Height difference:")), 0, 1, 6, 7,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        table.attach(gtk.Label(slopedata['height_difference']), 1, 2, 6, 7,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        table.attach(gtk.Label(_(u"Height gain:")), 0, 1, 7, 8,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        table.attach(gtk.Label(slopedata['height_gain']), 1, 2, 7, 8,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)

        self.s_comment = gtk.TextBuffer()
        self.s_comment.set_text(slopedata['comment'])
        textview = gtk.TextView(self.s_comment)
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.set_placement(gtk.CORNER_TOP_LEFT)
        sw.add(textview)
        sw.show_all()
        box.pack_start(gtk.Label(_(u"Note")+":"), False, False, 0)
        box.pack_start(sw, True, True, 0)

        box.show_all()

    def getslopeinfo(self):
        """Returns the slope infos: name, state, comment"""
        return {'name' : self.s_name.get_text(),
                'state' : self.s_state.get_text(),
                'author' : self.s_author.get_text(),
                'email' : self.s_email.get_text(),
                'comment' : self.s_comment.get_text(*self.s_comment.get_bounds())}

    def show_modal(self):
        """Shows the dialog and returns True if the value has to be chenged."""
        return self.run() == gtk.RESPONSE_ACCEPT

    def destroy(self):
        """Destroy current FormSlopeInfo"""
        gtk.Dialog.destroy(self)

class tab_label(gtk.HBox):
    """Notebook tab custom label with close button"""
    def __init__(self, page, title):
        gtk.HBox.__init__(self)
        self.label = gtk.Label(title)
        self.closebtn = gtk.Button()
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
        self.closebtn.set_image(image)
        self.closebtn.set_relief(gtk.RELIEF_NONE)
        self.pack_start(self.label, True, True)
        self.pack_end(self.closebtn, False, False)
        self.show_all()

class Plot(gtk.Window):
    """plot main class"""
    def __init__(self, parent, slope, plot_settings,
                 exportfunc, exportfunc_pdf, exportfunc_html):
        super(Plot, self).__init__()

        title = "CycloGraph"
        if slope.name:
            title += " - " + slope.name
        self.set_title(title)
        self.settings = plot_settings
        self.set_default_size(self.settings['width'], self.settings['height'])
        # set minimum height and width that plot can have
        self.set_geometry_hints(self, 300, 300)
        self.set_icon_from_file(ICON_FILENAME)

        menubar = gtk.MenuBar()
        filemenu = gtk.Menu()
        self.menu = gtk.MenuItem(_(u"_Options"))
        self.menu.set_submenu(filemenu)

        agr = gtk.AccelGroup()
        self.add_accel_group(agr)

        self.menu_item_save = gtk.ImageMenuItem(gtk.STOCK_SAVE, agr)
        key, mod = gtk.accelerator_parse("<Control>S")
        self.menu_item_save.add_accelerator("activate", agr,
                                            key, mod, gtk.ACCEL_VISIBLE)
        filemenu.append(self.menu_item_save)
        self.menu_item_savepdf = gtk.MenuItem(_(u"Save _PDF"))
        filemenu.append(self.menu_item_savepdf)
        self.menu_item_savehtml = gtk.MenuItem(_(u"Save _html"))
        filemenu.append(self.menu_item_savehtml)
        self.menu_item_olines = gtk.CheckMenuItem(_(u"_Orizzontal lines"))
        self.menu_item_olines.set_active(self.settings['olines'])
        filemenu.append(self.menu_item_olines)
        self.menu_item_3d = gtk.CheckMenuItem(_(u"3D _effect"))
        self.menu_item_3d.set_active(self.settings['3d'])
        filemenu.append(self.menu_item_3d)

        tmenu = gtk.Menu()
        tmenum = gtk.MenuItem(_(u"_Theme"))
        tmenum.set_submenu(tmenu)

        self.themeitems = []
        self.radioitem = {}
        self.thememanager = ThemeManager()
        for theme in self.thememanager.getthemeslist():
            if self.settings['theme'] == theme:
                self.theme1 = gtk.RadioMenuItem( None, theme)
            self.themeitems.append(theme)
        self.theme1.connect("activate", self.changetheme)
        for theme in self.themeitems:
            if self.settings['theme'] != theme:
                rmi = gtk.RadioMenuItem(self.theme1, theme)
                rmi.connect("activate", self.changetheme)
                tmenu.append(rmi)
                self.radioitem[theme] = rmi
            else:
                self.radioitem[theme] = self.theme1
                tmenu.append(self.theme1)
        self.theme1.set_active(True)

        filemenu.append(tmenum)

        rmenu = gtk.Menu()
        rmenum = gtk.MenuItem(_(u"Saving _resolution"))
        rmenum.set_submenu(rmenu)

        menu_item_r1 = gtk.RadioMenuItem( None, (u"800x600"))
        menu_item_r2 = gtk.RadioMenuItem(menu_item_r1, (u"1024x768"))
        menu_item_r3 = gtk.RadioMenuItem(menu_item_r1, (u"1280x800"))
        menu_item_r4 = gtk.RadioMenuItem(menu_item_r1, (u"1600x1200"))
        menu_item_r1.set_active(self.settings['width'] == 800)
        menu_item_r2.set_active(self.settings['width'] == 1024)
        menu_item_r3.set_active(self.settings['width'] == 1280)
        menu_item_r4.set_active(self.settings['width'] == 1600)
        menu_item_r1.connect("activate", self.res1)
        menu_item_r2.connect("activate", self.res2)
        menu_item_r3.connect("activate", self.res3)
        menu_item_r4.connect("activate", self.res4)

        rmenu.append(menu_item_r1)
        rmenu.append(menu_item_r2)
        rmenu.append(menu_item_r3)
        rmenu.append(menu_item_r4)

        filemenu.append(rmenum)

        menubar.append(self.menu)

        self.menu_item_save.connect("activate", self.saveimage)
        self.menu_item_savepdf.connect("activate", self.savepdf)
        self.menu_item_savehtml.connect("activate", self.savehtml)
        self.menu_item_olines.connect("activate", self.olines)
        self.menu_item_3d.connect("activate", self.change3d)

        darea = gtk.DrawingArea()
        darea.connect("expose-event", self.on_paint)

        self.myslope = slope
        self.devc = glal.DeviceContext()
        self.myslope.calc()
        self.exportfunc = exportfunc
        self.exportfunc_pdf = exportfunc_pdf
        self.exportfunc_html = exportfunc_html

        vbox = gtk.VBox(False, 2)
        vbox.pack_start(menubar, False, False, 0)
        vbox.pack_start(darea, True, True, 0)
        self.add(vbox)
        self.show_all()

    def on_paint(self, widget, event):
        """Draw graph"""
        gdkwin = widget.window
        devc = gdkwin.cairo_create()
        self.devc.init_draw_surf(devc, widget.get_allocation().width,
                                 widget.get_allocation().height)
        self.myslope.paint(self.settings, self.devc)

    def showplot(self):
        """ Show plot"""
        pass

    def saveimage(self, event):
        """ Save plot image"""
        self.exportfunc(self.myslope, self.settings)

    def savepdf(self, event):
        """ Save pdf report"""
        self.exportfunc_pdf(self.myslope, self.settings)

    def savehtml(self, event):
        """ Save html report"""
        self.exportfunc_html(self.myslope, self.settings)

    def change3d(self, event):
        """ If active switch 3d effect off, else turn it on."""
        self.settings['3d'] = not self.settings['3d']
        self.queue_draw()

    def olines(self, event):
        """ If active draws orizzontal lines"""
        self.settings['olines'] = not self.settings['olines']
        self.queue_draw()

    def changetheme(self, event):
        """Change the actual slope theme."""
        t = self.radioitem.items()
        for theme in t:
            if theme[1] == event:
                self.settings["theme"] = theme[0]
                self.queue_draw()
                break
    def res1(self, event=None):
        """Set 800x600"""
        self.settings['width'] = 800
        self.settings['height'] = 600
    def res2(self, event=None):
        """Set 1024x768"""
        self.settings['width'] = 1024
        self.settings['height'] = 768
    def res3(self, event=None):
        """Set 1280x800"""
        self.settings['width'] = 1280
        self.settings['height'] = 800
    def res4(self, event=None):
        """Set 1600x1200"""
        self.settings['width'] = 1600
        self.settings['height'] = 1200

FONT_TYP = {
            "300" : "light",
            "400" : "normal",
            "700" : "bold",
           }
class Preferences(gtk.Dialog):
    """Set program preferences"""
    def __init__(self, parent, pref, default_pref):
        gtk.Dialog.__init__(self, _(u"Preferences"), parent,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)

        self.colorlist = []

        for color in pref['colors']:
            components = [int(component)<<8 for component in color[4:-1].split(',')]
            self.colorlist.append(gtk.gdk.Color(*components))

        self.default = default_pref
        self.set_size_request(300, 480)
        notebook = gtk.Notebook()
        notebook.set_tab_pos(gtk.POS_TOP)

        frame = gtk.Frame()
        vbox = gtk.VBox(False, 0)
        vbox.set_border_width(5)
        hbox = gtk.HBox(False, 0)
        hbox.pack_start(vbox, False, False, 10)
        frame.add(hbox)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        label = gtk.Label("<b>"+_(u"Visualization")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)
        self.olines = gtk.CheckButton(_(u"Orizzontal lines"))
        vbox.pack_start(self.olines, False, False, 0)
        self.olines.set_active(pref['olines'])
        self.effect3d = gtk.CheckButton(_(u"3D effect"))
        vbox.pack_start(self.effect3d, False, False, 0)
        self.effect3d.set_active(pref['3d'])
        self.showinfo = gtk.CheckButton(_(u"Show info"))
        vbox.pack_start(self.showinfo, False, False, 0)
        self.showinfo.set_active(pref['sinfo'])

        label = gtk.Label("<b>"+_(u"Resolution")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)
        self.res = gtk.combo_box_new_text()
        self.res.append_text('800 x 600')
        self.res.append_text('1024 x 768')
        self.res.append_text('1280 x 800')
        self.res.append_text('1600 x 1200')
        self.res.set_active(0)
        if pref['width'] == 1600:
            self.res.set_active(3)
        elif pref['width'] == 1280:
            self.res.set_active(2)
        elif pref['width'] == 1024:
            self.res.set_active(1)
        vbox.pack_start(self.res, False, False, 0)

        label = gtk.Label("<b>"+_(u"Theme")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)
        self.theme = gtk.combo_box_new_text()
        current = 0
        for theme in pref['themelist']:
            self.theme.append_text(theme)
            if theme == pref['theme']:
                self.theme.set_active(current)
            current += 1
        vbox.pack_start(self.theme, False, False, 0)

        label = gtk.Label("<b>"+_(u"Kml server")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)
        self.serv = gtk.combo_box_new_text()
        self.serv.append_text('geonames.org')
        self.serv.append_text('earthtools.org')
        self.serv.append_text('usgs.net')
        self.serv.append_text('google elevation')
        self.serv.set_active(0)
        if pref['serv'] == 'usgs.net':
            self.serv.set_active(2)
        elif pref['serv'] == 'earthtools.org':
            self.serv.set_active(1)
        elif pref['serv'] == 'google elevation':
            self.serv.set_active(3)
        vbox.pack_start(self.serv, False, False, 0)

        label = gtk.Label("<b>"+_(u"Latest version")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)
        self.vcheck = gtk.CheckButton(_(u"Check online"))
        vbox.pack_start(self.vcheck, False, False, 0)
        self.vcheck.set_active(pref['vcheck'])

        notebook.append_page(frame, gtk.Label("General"))

        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_NONE)
        vbox = gtk.VBox(False, 0)
        vbox.set_border_width(5)
        hbox = gtk.HBox(False, 0)
        hbox.pack_start(vbox, False, False, 10)
        frame.add(hbox)

        label = gtk.Label("<b>"+_(u"Description")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)

        desc = pref['fdesc']
        self.fdesc = gtk.Button(desc['des']+' | '+str(desc["dim"]))
        self.fdesc.desc = desc
        self.fdesc.connect("clicked", self.font_button_clicked)
        vbox.pack_start(self.fdesc, False, False, 0)

        label = gtk.Label("<b>"+_(u"Gradient")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)

        desc = pref['fgrad']
        self.fgrad = gtk.Button(desc['des']+' | '+str(desc["dim"]))
        self.fgrad.desc = desc
        self.fgrad.connect("clicked", self.font_button_clicked)
        vbox.pack_start(self.fgrad, False, False, 0)

        label = gtk.Label("<b>"+_(u"Title")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)

        desc = pref['ftitle']
        self.ftitle = gtk.Button(desc['des']+' | '+str(desc["dim"]))
        self.ftitle.desc = desc
        self.ftitle.connect("clicked", self.font_button_clicked)
        vbox.pack_start(self.ftitle, False, False, 0)

        vbox.show_all()
        notebook.append_page(frame, gtk.Label("Fonts"))

        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_NONE)
        vbox = gtk.VBox(False, 0)
        vbox.set_border_width(5)
        frame.add(vbox)
        self.table = gtk.Table(6, 2, True)
        vbox.pack_start(self.table, True, True, 0)
        self.table.show()

        self.tick = 0.1
        self.levlist = []
        for i in range(len(pref['levels'])):
            self.levlist.append(gtk.SpinButton())
            self.levlist[i].set_range(0.0, 90.0)
            self.levlist[i].set_value(pref['levels'][i])
            self.levlist[i].set_increments(self.tick, 1)
            self.levlist[i].set_numeric(True)
            self.levlist[i].set_digits(1)
            self.levlist[i].connect("value_changed", self.changed_level)

            hbx = gtk.HBox()
            hbx.pack_start(gtk.Label(">"), False, False, 5)
            hbx.pack_start(self.levlist[i], False, False, 5)
            hbx.pack_start(gtk.Label("%"), False, False, 5)
            self.table.attach(hbx, 0, 1, i, i+1, 0)

        self.button1 = gtk.Button("")
        self.attach_button(self.button1, 0, 1)
        self.button2 = gtk.Button()
        self.attach_button(self.button2, 1, 2)
        self.button3 = gtk.Button("")
        self.attach_button(self.button3, 2, 3)
        self.button4 = gtk.Button("")
        self.attach_button(self.button4, 3, 4)
        self.button5 = gtk.Button("")
        self.attach_button(self.button5, 4, 5)
        self.button6 = gtk.Button("")
        self.attach_button(self.button6, 5, 6)
        self.button7 = gtk.Button("")
        self.attach_button(self.button7, 6, 7)

        vbox.show_all()
        notebook.append_page(frame, gtk.Label("Gradient colors"))

        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_NONE)
        vbox = gtk.VBox(False, 0)
        vbox.set_border_width(5)
        hbox = gtk.HBox(False, 0)
        hbox.pack_start(vbox, False, False, 10)
        frame.add(hbox)

        label = gtk.Label("<b>"+_(u"Name")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)

        self.aname = gtk.Entry()
        self.aname.set_text(pref['aname'])
        vbox.pack_start(self.aname, False, False, 0)

        label = gtk.Label("<b>"+_(u"E-mail")+"</b>")
        label.set_use_markup(True)
        hhbox = gtk.HBox(False, 0)
        hhbox.pack_start(label, False, False, 0)
        vbox.pack_start(hhbox, False, False, 5)

        self.amail = gtk.Entry()
        self.amail.set_text(pref['amail'])
        vbox.pack_start(self.amail, False, False, 0)

        vbox.show_all()
        notebook.append_page(frame, gtk.Label("Author"))
        notebook.show_all()

        bbox = gtk.HBox()
        button_k = gtk.Button(None, gtk.STOCK_OK)
        button_c = gtk.Button(None, gtk.STOCK_CANCEL)
        button_d = gtk.Button(_(u"Restore Default"))
        bbox.pack_end(button_k, False, False, 5)
        bbox.pack_end(button_c, False, False, 5)
        bbox.pack_start(button_d, False, False, 5)
        button_k.connect("clicked", self.ok_clicked)
        button_d.connect("clicked", self.restore_default)
        button_c.connect("clicked", self.destroy)

        box = gtk.VBox(False, 0)
        box.pack_start(notebook, True, True, 5)
        box.pack_start(bbox, False, False, 5)
        self.vbox.pack_start(box)
        self.vbox.show_all()

    def changed_level(self, event):
        self.levlist[0].set_range(0.0, self.levlist[1].get_value() - self.tick)
        for i in range(1, len(self.levlist)-1):
            self.levlist[i].set_range(self.levlist[i-1].get_value() + self.tick,
                                      self.levlist[i+1].get_value() - self.tick)
        self.levlist[len(self.levlist)-1].set_range(
              self.levlist[len(self.levlist)-2].get_value() + self.tick, 90.0)

    def ok_clicked(self, event):
        self.response(True)

    def restore_default(self, event):
        """Restore preferences to default value"""
        self.olines.set_active(bool(self.default['orizzontal lines']))
        self.effect3d.set_active(bool(self.default['effect-3d']))
        self.showinfo.set_active(bool(self.default['slopeinfo']))
        self.vcheck.set_active(bool(self.default['vcheck']))
        self.res.set_active(0)
        if self.default['width'] == 1600:
            self.res.set_active(3)
        elif self.default['width'] == 1280:
            self.res.set_active(2)
        elif self.default['width'] == 1024:
            self.res.set_active(1)
        current = 0
        model = self.theme.get_model()
        for text in model:
            if text[0] == self.default['theme']:
                self.theme.set_active(current)
            current += 1
        self.serv.set_active(0)
        if self.default['server'] == 'usgs.net':
            self.serv.set_active(2)
        elif self.default['server'] == 'earthtools.org':
            self.serv.set_active(1)
        elif self.default['server'] == 'google elevation':
            self.serv.set_active(3)
        self.fdesc = gtk.Button(self.default['font-des 1']+' | '+self.default['font-dim 1'])
        self.fdesc.desc = {
                               "des" : self.default['font-des 1'],
                               "typ" : self.default['font-typ 1'],
                               "dim" : self.default['font-dim 1']
                           }
        self.fgrad = gtk.Button(self.default['font-des g']+' | '+self.default['font-dim g'])
        self.fgrad.desc = {
                               "des" : self.default['font-des g'],
                               "typ" : self.default['font-typ g'],
                               "dim" : self.default['font-dim g']
                           }
        self.ftitle = gtk.Button(self.default['font-des t']+' | '+self.default['font-dim t'])
        self.ftitle.desc = {
                               "des" : self.default['font-des t'],
                               "typ" : self.default['font-typ t'],
                               "dim" : self.default['font-dim t']
                           }

        self.aname.set_text(self.default['authorname'])
        self.amail.set_text(self.default['authormail'])
        colorlist = [self.default["color "+str(i)] for i in range(1, 8)]
        self.colorlist = []
        for color in colorlist:
            components = [int(component)<<8 for component in color[4:-1].split(',')]
            self.colorlist.append(gtk.gdk.Color(*components))
        self.button1.modify_bg(gtk.STATE_NORMAL, self.colorlist[0])
        self.button1.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[0])
        self.button2.modify_bg(gtk.STATE_NORMAL, self.colorlist[1])
        self.button2.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[1])
        self.button3.modify_bg(gtk.STATE_NORMAL, self.colorlist[2])
        self.button3.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[2])
        self.button4.modify_bg(gtk.STATE_NORMAL, self.colorlist[3])
        self.button4.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[3])
        self.button5.modify_bg(gtk.STATE_NORMAL, self.colorlist[4])
        self.button5.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[4])
        self.button6.modify_bg(gtk.STATE_NORMAL, self.colorlist[5])
        self.button6.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[5])
        self.button7.modify_bg(gtk.STATE_NORMAL, self.colorlist[6])
        self.button7.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[6])
        levellist = [self.default["level "+str(i)] for i in range(1, 8)]
        for k in range(len(self.levlist)):
            i = len(self.levlist) - k-1
            self.levlist[i].set_value(float(levellist[i]))

    def font_button_clicked(self, button):
        """If a font button is clicked show a font dialog to change font"""
        dlg = gtk.FontSelectionDialog(_(u"Select Font"))
        dlg.set_font_name(button.desc["des"]+' '+button.desc["typ"].capitalize()+' '+str(button.desc["dim"]))
        dlg.set_icon_from_file(ICON_FILENAME)
        result = dlg.run()
        if result == gtk.RESPONSE_OK:
            font = pango.FontDescription(dlg.get_font_name())
            res = str(int(font.get_weight()))
            dlg.destroy()
            if FONT_TYP.has_key(res):
                weight = FONT_TYP[res]
            else:
                weight = "normal"
            button.desc = {
                        "des" : font.get_family(),
                        "typ" : weight,
                        "dim" : str(font.get_size()/pango.SCALE)
                      }
            button.set_label(font.get_family()+' | '+str(font.get_size()/pango.SCALE))
        dlg.destroy()

    def attach_button(self, button, posx, posy):
        """bind button with changecolor function and set it in the table"""
        button.num = posx
        button.modify_bg(gtk.STATE_NORMAL, self.colorlist[posx])
        button.modify_bg(gtk.STATE_PRELIGHT, self.colorlist[posx])
        self.table.attach(button, 1, 2, posx, posy,
                          gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 5)
        button.connect("clicked", self.changecolor)

    def changecolor(self, widget):
        """Change the color used by plot to fill different classes of gradients
        """
        csd = gtk.ColorSelectionDialog(_(u"Select color"))
        csd.set_icon_from_file(ICON_FILENAME)
        csd.colorsel.set_current_color(self.colorlist[widget.num])
        response = csd.run()

        if response == gtk.RESPONSE_OK:
            color = csd.colorsel.get_current_color()
            self.colorlist[widget.num] = color
            widget.modify_bg(gtk.STATE_NORMAL, color)
            widget.modify_bg(gtk.STATE_PRELIGHT, color)
        csd.destroy()

    def show_modal(self):
        """Shows the dialog and returns True if the values have to be chenged."""
        return self.run() == 1

    def getconfig(self):
        """Returns selected config options"""
        res = self.res.get_active_text().split(' ')
        levels = []
        for i in range(len(self.levlist)):
            levels.append(self.levlist[i].get_value())
        settings = {
                    'olines': self.olines.get_active(),
                    'theme' : self.theme.get_active_text(),
                    'sinfo' : self.showinfo.get_active(),
                    '3d'    : self.effect3d.get_active(),
                    'vcheck': self.vcheck.get_active(),
                    'aname' : self.aname.get_text(),
                    'amail' : self.amail.get_text(),
                    'fdesc' : self.fdesc.desc,
                    'ftitle': self.ftitle.desc,
                    'fgrad' : self.fgrad.desc,
                    'width' : res[0],
                    'height': res[2],
                    'serv'  : self.serv.get_active_text(),
                    'colors': ['rgb(%d,%d,%d)' % (color.red>>8, color.green>>8,
                            color.blue>>8) for color in self.colorlist],
                    'levels': levels
                   }
        return settings

    def destroy(self, event=None):
        """Destroy current Preferences"""
        gtk.Dialog.destroy(self)

class Map(gtk.Dialog):
    """Show map of slopes"""
    def __init__(self, parent, slopename, stringc, savefnct):
        gtk.Window.__init__(self)

        self.set_size_request(640, 480)#FIXME:take webpage dimension
        self.set_title(_(u"Map")+" - "+ slopename)
        self.set_icon_from_file(ICON_FILENAME)
        self.set_resizable(True)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)

        self.savefnct = savefnct

        self.view = webkit.WebView()
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        #sw.set_placement(gtk.CORNER_TOP_LEFT)
        sw.add(self.view)

        swframe = gtk.Frame()
        swframe.add(sw)

        obox = gtk.HBox()
        obox.pack_start(swframe, True, True, 0)

        bbox = gtk.HBox()
        button_s = gtk.Button(None, gtk.STOCK_SAVE)
        bbox.pack_end(button_s, False, False, 5)
        button_s.connect("clicked", self.onsave)

        box = gtk.VBox(False, 0)
        box.pack_start(obox, True, True, 0)
        box.pack_start(bbox, False, False, 5)
        self.vbox.pack_start(box)
        self.show_all()
        MAP_SCRIPT = ""
        try:
            fid = open(os.path.join(os.path.dirname(__file__),
                                         "../map.html"), 'rb')
            for line in fid:
                MAP_SCRIPT += line
            fid.close()
        except IOError:
            print("map.html not loaded\n")
            return
        HTML_MAP = MAP_SCRIPT % (stringc,)
        self.view.load_html_string(HTML_MAP, "file:///")

    def onsave(self, event):
        """ Save call"""
        self.savefnct(self)

    def saveimage(self, filepath):
        """ Save map image"""
        winw, winh = self.get_size()
        vieww, viewh = self.view.size_request()
        diff = (winw - vieww) / 2
        colormap = self.get_colormap()
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, vieww, viewh)
        pixbuf = pixbuf.get_from_drawable(self.window, colormap,
                                          diff, diff, 0, 0, vieww, viewh)

        fformat = filepath.rsplit('.', 1)[1]

        if fformat.lower() == 'png':
            pixbuf.save(filepath, 'png')
        if fformat.lower() == 'bmp':
            pixbuf.save(filepath, "bmp")
        if fformat.lower() == 'jpg':
            pixbuf.save(filepath, "jpeg", {"quality":"100"})


class Create_kml(gtk.Dialog):
    """Create kml routes"""
    MODE = {_(u'Open Route Service') : 'ORS',
            _(u'OSRM') : 'OSRM',
            _(u"Draw on the map") : 'DRAW'}
    def __init__(self, parent, downloadfunc, savefunc):
        gtk.Window.__init__(self)

        self.set_size_request(640, 480)#FIXME:take webpage dimension
        self.set_title(_(u"Create Kml"))
        self.set_icon_from_file(ICON_FILENAME)
        self.set_resizable(True)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
        self.downloadfunc = downloadfunc
        self.savefunc = savefunc
        self.handler = None
        self.mode = 'ORS'

        self.view = webkit.WebView()
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.set_placement(gtk.CORNER_TOP_LEFT)
        sw.add(self.view)

        store = gtk.ListStore(str)
        store.append([_(u'Open Route Service')])
        store.append([_(u'OSRM')])
        store.append([_(u'Draw on the map')])

        treeview = gtk.TreeView(store)
        treeview.connect("cursor-changed", self.mode_changed)

        renderertext = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Views", renderertext, text=0)
        column.set_sort_column_id(0)
        treeview.append_column(column)

        frame = gtk.Frame(_(u'Modes'))
        frame.add(treeview)

        swframe = gtk.Frame()
        swframe.add(sw)
        treeview.set_cursor(0)

        treeview.set_headers_visible(False)
        obox = gtk.HBox()
        obox.pack_start(frame, False, True, 5)
        obox.pack_start(swframe, True, True, 5)

        bbox = gtk.HBox()
        button_k = gtk.Button(None, gtk.STOCK_OK)
        button_s = gtk.Button(None, gtk.STOCK_SAVE)
        button_c = gtk.Button(None, gtk.STOCK_CANCEL)
        bbox.pack_end(button_k, False, False, 5)
        bbox.pack_end(button_s, False, False, 5)
        bbox.pack_end(button_c, False, False, 5)
        button_k.connect("clicked", self.import_kml)
        button_s.connect("clicked", self.save_kml)
        button_c.connect("clicked", self.cancel)

        box = gtk.VBox(False, 0)
        box.pack_start(obox, True, True, 5)
        box.pack_start(bbox, False, False, 5)
        self.vbox.pack_start(box)
        self.show_all()
        self.webview_init()

    def webview_init(self):
        """ Check mode and set webview accordingly"""
        if self.mode == 'ORS':
            self.view.open("http://openrouteservice.org/")
        elif self.mode == 'OSRM':
            self.view.open("http://map.project-osrm.org/")
        elif self.mode == 'DRAW':
            HTML_DRAW = ""
            try:
                fid = open(os.path.join(os.path.dirname(__file__),"../openlayers.html"), 'rb')
                for line in fid:
                    HTML_DRAW += line
                fid.close()
            except IOError:
                print("openlayers.html not loaded\n")
                exit(1)
            self.view.load_html_string(HTML_DRAW.replace("file://./draw_on_map.js","file://"+os.path.join(os.path.dirname(__file__), "../draw_on_map.js")), "file:///")

    def mode_changed(self, widget):
        """ Change current mode"""
        selection = widget.get_selection()
        if selection is None:
            return
        model, treeiter = selection.get_selected()
        if treeiter is None:
            return
        newmode = self.MODE[model.get_value(treeiter, 0)]
        if newmode != self.mode:
            self.mode = newmode
            self.webview_init()

    def import_kml(self, event):
        """ Import route in CycloGraph"""
        self.download()
        self.response(True)

    def get_ors(self):
        """ Returns gpx content from ORS service"""
        self.view.execute_script("oldtitle=document.title;document.title=document.getElementById('gpx').innerHTML")
        gpxcontent = unicode(self.view.get_main_frame().get_title())
        self.view.execute_script('document.title=oldtitle;')

        from xml.sax.saxutils import unescape
        return StringIO.StringIO(unescape(gpxcontent))

    def get_osrm(self):
        """ Returns gpx content from ORSM service"""
        self.view.execute_script("oldtitle=document.title;document.title=document.getElementsByClassName('text-link')[1].getAttribute('onclick')")
        gpxcontent = unicode(self.view.get_main_frame().get_title())
        url = gpxcontent.split('\'')
        self.view.execute_script('document.title=oldtitle;')
        return self.downloadfunc(url[1])

    def get_draw(self):
        """ Returns kml content from DRAW mode"""
        self.view.execute_script("oldtitle=document.title;document.title=createKml()")
        kmlstring = self.view.get_main_frame().get_title()
        self.view.execute_script('document.title=oldtitle;')
        return StringIO.StringIO(kmlstring)

    def download(self):
        """ Download selected route"""
        if self.mode == 'ORS':
            self.handler = self.get_ors()
        elif self.mode == 'OSRM':
            self.handler = self.get_osrm()
        elif self.mode == 'DRAW':
            self.handler = self.get_draw()

    def save_kml(self, event):
        """Save selected route to file"""
        self.download()
        if not self.handler:
            return
        self.savefunc(self.handler, self.mode)

    def cancel(self, event):
        """ Cancel create kml dialog"""
        self.destroy()

    def show_modal(self):
        """Shows the dialog and returns True if the value has to be changed."""
        return self.run() == 1

FONT_TYP = {
            "300" : "light",
            "400" : "normal",
            "700" : "bold",
           }

def preferences(parent, cgpref, defaultpref):
    """ Show Preference dialog"""
    pref = Preferences(parent, cgpref, defaultpref)
    pref_conf = {}
    if pref.show_modal():
        pref_conf = pref.getconfig()
    pref.destroy()
    return pref_conf

def formslopeinfo(parent, slopedata):
    """ Show slope's info dialog"""
    dlg = FormSlopeInfo(parent, slopedata)
    if dlg.show_modal():
        data = dlg.getslopeinfo()
        dlg.destroy()
        return data
    else:
        dlg.destroy()
        return {}

def FileDialog(parent, title, dirname, filename, other, file_types, rw, fd_open, fd_save):
    """ Show file dialog for open or save slopes"""
    if rw == fd_open:
        dlg = gtk.FileChooserDialog(title, parent, gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    elif rw == fd_save:
        dlg = gtk.FileChooserDialog(title, parent, gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dlg.set_do_overwrite_confirmation(True)
        if filename:
            dlg.set_current_name(filename)
    for elem in file_types:
        ffilter = gtk.FileFilter()
        ffilter.set_name(elem[0])
        for pattern in elem[1].split(' '):
            ffilter.add_pattern(pattern)
            ffilter.add_pattern(pattern.upper())
            #print(pattern)
        #print("----")
        dlg.add_filter(ffilter)
    head, tail = os.path.split(dirname)
    dlg.set_current_folder(head)
    if tail:
        dlg.set_current_name(tail)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        filename = dlg.get_filename()
        dlg.destroy()
        return unicode(filename)
    else:
        dlg.destroy()
        return None

class ProgressDialog(gtk.Dialog):
    """Progress dialog for import Kml function"""
    def __init__(self, title, label, limit):
        gtk.Dialog.__init__(self, title, None,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
                     (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))

        self.set_size_request(330, 150)
        box = gtk.VBox()
        self.limit = limit
        self.label = gtk.Label(label)
        self.pbar = gtk.ProgressBar()
        align = gtk.Alignment(0.5, 0.5, 0.5, 0.5)
        align.show()
        align.add(self.pbar)
        self.pbar.show()
        self.pbar.set_fraction(0.0)
        self.pbar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
        self.pbar.set_text("0"+" %")

        box.pack_start(self.label, False, False, 10)
        box.pack_start(align, False, False, 5)

        self.wanttoabort = False
        self.connect("response", self.abort)

        self.vbox.pack_start(box)
        box.show_all()

        self.show_all()

    def update(self, value):
        """Update the progress shown and return if user want to abort."""
        self.pbar.set_fraction(value/self.limit)
        self.pbar.set_text(str(value/10)+" %")
        return self.wanttoabort

    def abort(self, dialog, response):
        """When the abort button is pressed this method is called and it updates
        the status of this object to take track of user action"""
        self.wanttoabort = (response in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT))

    def destroy(self):
        """Destroy current FormSlopeInfo"""
        gtk.Dialog.destroy(self)

class NumcpsDialog(gtk.Dialog):
    """ask cps number you want to import"""
    def __init__(self, max_num_cps, max_length, file_ext):
        gtk.Dialog.__init__(self, _(u"number of checkpoints"), None,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR, None)

        self.set_size_request(330, 250)
        box = gtk.VBox()
        self.notebook = gtk.Notebook()
        self.notebook.set_show_tabs(False)

        self.connect("delete-event", self.abort)


        frame = gtk.Frame()
        vbox_1 = gtk.VBox()
        
        label = gtk.Label(_(u"Choose the type of import you want")) 
        vbox_1.pack_start(label, False, False, 15)

        hboxr = gtk.HBox()
        vboxr = gtk.VBox()
        self.button1 = gtk.RadioButton(None, _(u"Automatic"))
        self.button1.set_active(True)
        vboxr.pack_start(self.button1, False, False, 0)

        self.button2 = gtk.RadioButton(self.button1)
        self.button2.set_label(_(u"Select the number of checkpoints"))
        vboxr.pack_start(self.button2, False, False, 0)

        self.button3 = gtk.RadioButton(self.button1)
        self.button3.set_label(_(u"Select the distance rate"))
        vboxr.pack_start(self.button3, False, False, 0)

        self.button4 = gtk.RadioButton(self.button1)
        self.button4.set_label(_(u"Use every point (discouraged)"))
        vboxr.pack_start(self.button4, False, False, 0)

        hboxr.pack_start(vboxr, False, False, 30)

        bbox = gtk.HBox()
        button_n = gtk.Button(_(u"Next"))
        bbox.pack_end(button_n, False, False, 5)
        button_n.connect("clicked", self.next_clicked)
        vbox_1.pack_end(bbox, False, False, 10)
        vbox_1.pack_end(hboxr, False, False, 5)

        frame.add(vbox_1)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        self.notebook.append_page(frame, gtk.Label("1"))

        frame = gtk.Frame()
        label = gtk.Label(_(u"The %s file contains %d points.\nChoose how many will be imported.") % (file_ext, max_num_cps))
        self.gpxnum = gtk.HScale()
        self.gpxnum.set_range(0, max_num_cps)
        self.gpxnum.set_increments(1, 1)
        self.gpxnum.set_digits(0)
        self.gpxnum.set_size_request(160, 38)
        self.gpxnum.set_value(0)

        box.pack_start(label, False, False, 5)
        box.pack_start(self.gpxnum, False, False, 5)

        bbox = gtk.HBox()
        button_n = gtk.Button(_(u"Finish"))
        button_c = gtk.Button(_(u"Back"))
        bbox.pack_end(button_n, False, False, 5)
        bbox.pack_end(button_c, False, False, 5)
        button_n.connect("clicked", self.next_clicked)
        button_c.connect("clicked", self.back_clicked)
        box.pack_end(bbox, False, False, 10)

        frame.add(box)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        self.notebook.append_page(frame, gtk.Label("2"))

        frame = gtk.Frame()
        boxdist = gtk.VBox()
        label = gtk.Label(_(u"The %s file contains a route of %d Km.\nChoose what range you want between points.") % (file_ext, max_length))
        self.gpxdist = gtk.HScale()
        self.gpxdist.set_range(0, int(max_length))
        self.gpxdist.set_increments(1, 1)
        self.gpxdist.set_digits(0)
        self.gpxdist.set_size_request(160, 38)
        self.gpxdist.set_value(0)

        boxdist.pack_start(label, False, False, 5)
        boxdist.pack_start(self.gpxdist, False, False, 5)

        bbox = gtk.HBox()
        button_n = gtk.Button(_(u"Finish"))
        button_c = gtk.Button(_(u"Back"))
        bbox.pack_end(button_n, False, False, 5)
        bbox.pack_end(button_c, False, False, 5)
        button_n.connect("clicked", self.next_clicked)
        button_c.connect("clicked", self.back_clicked)
        boxdist.pack_end(bbox, False, False, 10)

        frame.add(boxdist)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        self.notebook.append_page(frame, gtk.Label("3"))

        self.notebook.set_current_page(0)
        self.notebook.show_all()

        carea = self.get_content_area()
        carea.pack_start(self.notebook, True, True, 0)

        self.show_all()

    def next_clicked(self, event):
        "Show next page"
        if self.notebook.get_current_page() == 0:
            if self.button1.get_active():
                self.response(True)
            elif self.button2.get_active():
                self.notebook.set_current_page(1)
                self.notebook.show_all()
            elif self.button3.get_active():
                self.notebook.set_current_page(2)
                self.notebook.show_all()
            elif self.button4.get_active():
                self.response(True)
        else:
            self.response(True)

    def back_clicked(self, event):
        "Show initial page"       
        self.notebook.set_current_page(0)
        self.notebook.show_all()

    def destroy(self):
        """Destroy current dialog"""
        gtk.Dialog.destroy(self)

    def abort(self, event=None, param=None):
        """Abort import"""
        self.response(False)
        gtk.Dialog.destroy(self)

def geterrnoroute(parent):
    """Get an error message explaining that you should have selected one route"""
    mdlg = gtk.MessageDialog(parent,
            gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
            gtk.BUTTONS_OK, _(u"Specify one route first."))
    mdlg.set_title(_(u'No route'))
    mdlg.run()
    mdlg.destroy()

def managecperr(parent):
    """Get an error message explaining that you should have inserted altitude and distance"""
    mdlg = gtk.MessageDialog(parent,
            gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
            gtk.BUTTONS_OK, _("Distance and altitude are required."))
    mdlg.set_title(_("Form incomplete"))
    mdlg.run()
    mdlg.destroy()

def mapcoorderr(parent):
    """Get an error message explaining that there are no coords in the slope"""
    mdlg = gtk.MessageDialog(parent,
            gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
            gtk.BUTTONS_OK, _("there aren't coordinates associated with this slope."))
    mdlg.set_title(_("No coordinates"))
    mdlg.run()
    mdlg.destroy()

def geterrnocp(parent):
    """Get an error message explaining that you should have at least 2 cps to plot"""
    mdlg = gtk.MessageDialog(parent,
            gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR,
            gtk.BUTTONS_OK, _(u"""There aren't enougth check points to plot.
 At least two points are required to plot."""))
    mdlg.set_title(_(u'Not enough check points'))
    mdlg.run()
    mdlg.destroy()

def numcpsdlg(max_num_cps, max_length, file_ext):
    """ask what number of cps in a gpx or tcx files have to be imported"""
    dlg = NumcpsDialog(max_num_cps, max_length, file_ext)
    response = dlg.run()
    if response == False:
        return (-1, 0)
    if dlg.button1.get_active():
        dlg.destroy()
        return (0, -1)
    elif dlg.button2.get_active():
        val = dlg.gpxnum.get_value()
        dlg.destroy()
        return (0, int(val))
    elif dlg.button3.get_active():
        val = dlg.gpxdist.get_value()
        dlg.destroy()
        return (1, int(val))
    elif dlg.button4.get_active():
        dlg.destroy()
        return (0, max_num_cps)

def save_changes(parent, filename):
    """Show a message if you close a modified and not saved slope"""
    response = None

    text = "<b>"+(_(u"Save changes to %s before closing?") % filename)+"</b>"
    secondary_text = _(u"If you don't save, all your changes will be "
                       "permanently lost.")
    mdlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
                          gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, text)
    mdlg.set_title(_(u'Save changes'))
    mdlg.set_markup(text)
    mdlg.format_secondary_text(secondary_text)
    mdlg.add_buttons(_(u'Close without saving'), gtk.RESPONSE_REJECT,
                  gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE,
                  gtk.RESPONSE_ACCEPT)
    mdlg.set_default_response(gtk.RESPONSE_ACCEPT)
    response = mdlg.run()
    mdlg.destroy()
    if response == gtk.RESPONSE_ACCEPT:
        return 'SAVE'
    if response == gtk.RESPONSE_REJECT:
        return 'DISCARD'
    if response == gtk.RESPONSE_CANCEL:
        return 'CANCEL'
    return response

# vim:sw=4:softtabstop=4:expandtab
